<?php
    
    namespace artbox\core\controllers;
    
    use artbox\core\models\PageCategory;
    use Yii;
    use artbox\core\models\Page;
    use artbox\core\models\PageSearch;
    use yii\filters\AccessControl;
    use yii\helpers\ArrayHelper;
    use yii\web\Controller;
    use yii\web\NotFoundHttpException;
    use yii\filters\VerbFilter;
    
    /**
     * PageController implements the CRUD actions for Page model.
     */
    class PageController extends Controller
    {
        /**
         * @return bool|string
         */
        public function getViewPath()
        {
            return \Yii::getAlias('@artbox/core/views/page');
        }
        /**
         * @inheritdoc
         */
        public function behaviors()
        {
            return [
                'access' => [
                    'class' => AccessControl::className(),
                    'rules' => [
                        [
                            'actions' => [
                                'login',
                                'error',
                            ],
                            'allow'   => true,
                        ],
                        [
                            'allow' => true,
                            'roles' => [ '@' ],
                        ],
                    ],
                ],
                'verbs'  => [
                    'class'   => VerbFilter::className(),
                    'actions' => [],
                ],
            ];
        }
        
        /**
         * Lists all Page models.
         *
         * @return mixed
         */
        public function actionIndex()
        {
            $searchModel = new PageSearch();
            $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
            
            return $this->render(
                'index',
                [
                    'searchModel'  => $searchModel,
                    'dataProvider' => $dataProvider,
                ]
            );
        }
        
        /**
         * Displays a single Page model.
         *
         * @param integer $id
         *
         * @return mixed
         */
        public function actionView($id)
        {
            return $this->render(
                'view',
                [
                    'model' => $this->findModel($id),
                ]
            );
        }
        
        /**
         * Creates a new Page model.
         * If creation is successful, the browser will be redirected to the 'view' page.
         *
         * @return mixed
         */
        public function actionCreate()
        {
            $model = new Page();
            $model->generateLangs();
            
            $model->categoryIds = [];
            
            if ($model->loadWithLangs(\Yii::$app->request)) {
                if ($model->saveWithLangs()) {
                    
                    $categories = PageCategory::find()
                                              ->where([ 'id' => \Yii::$app->request->post('categoryIds') ])
                                              ->all();
                    
                    $model->linkMany('categories', $categories);
                    
                    $model->saveImages(\Yii::$app->request->post());
                    
                    return $this->redirect(
                        [
                            'view',
                            'id' => $model->id,
                        ]
                    );
                }
            }
            return $this->render(
                'create',
                [
                    'model'      => $model,
                    'modelLangs' => $model->modelLangs,
                ]
            );
        }
        
        /**
         * Updates an existing Page model.
         * If update is successful, the browser will be redirected to the 'view' page.
         *
         * @param integer $id
         *
         * @return mixed
         */
        public function actionUpdate($id)
        {
            $model = $this->findModel($id);
            $model->generateLangs();
            
            $model->categoryIds = ArrayHelper::map(
                $model->categories,
                'id',
                'lang.title'
            );
            
            if ($model->loadWithLangs(\Yii::$app->request)) {
                if ($model->saveWithLangs()) {
                    
                    $categories = PageCategory::find()
                                              ->where([ 'id' => \Yii::$app->request->post('categoryIds') ])
                                              ->all();
                    
                    $model->linkMany('categories', $categories);
                    
                    $model->saveImages(\Yii::$app->request->post());
                    
                    return $this->redirect(
                        [
                            'view',
                            'id' => $model->id,
                        ]
                    );
                }
            }
            return $this->render(
                'update',
                [
                    'model'      => $model,
                    'modelLangs' => $model->modelLangs,
                ]
            );
        }
        
        /**
         * Deletes an existing Page model.
         * If deletion is successful, the browser will be redirected to the 'index' page.
         *
         * @param integer $id
         *
         * @return mixed
         */
        public function actionDelete($id)
        {
            $this->findModel($id)
                 ->delete();
            
            return $this->redirect([ 'index' ]);
        }
        
        /**
         * Finds the Page model based on its primary key value.
         * If the model is not found, a 404 HTTP exception will be thrown.
         *
         * @param integer $id
         *
         * @return array|\yii\db\ActiveRecord|Page
         * @throws NotFoundHttpException if the model cannot be found
         */
        protected function findModel($id)
        {
            $model = Page::find()
                         ->with(
                             [
                                 'lang',
                                 'categories.lang',
                             ]
                         )
                         ->where([ 'id' => $id ])
                         ->one();
            
            if ($model !== null) {
                return $model;
            } else {
                throw new NotFoundHttpException('The requested page does not exist.');
            }
        }
    }
